@@ -4,8 +4,6 @@
 
 from __future__ import annotations
 
-import hashlib
-import json
 import typing
 from collections.abc import Callable
 from datetime import datetime
@@ -127,6 +125,15 @@ class ElasticDataSource(
         }
         return data_object.attributes | to_ones_dict
 
+    def _convert_data_objects_in_update_to_dict(self, dict_: Dict) -> Dict:
+        ret = {}
+        for k, v in dict_.items():
+            if isinstance(v, DataObject):
+                ret[k] = self._convert_to_one_relation(v)
+            else:
+                ret[k] = v
+        return ret
+
     def _convert_to_one_relation(
         self,
         one_relation: DataObject | None
@@ -148,15 +155,6 @@ class ElasticDataSource(
             ret[prefix + '_' + k] = v
         return ret
 
-    def _add_updated(self, dict_: Dict) -> Dict:
-        return {**dict_, 'tol_updated_at': datetime.now().isoformat()}
-
-    def _add_checksum(self, dict_: Dict) -> Dict:
-        dhash = hashlib.sha256()
-        encoded = json.dumps(dict_, sort_keys=True, default=str).encode()
-        dhash.update(encoded)
-        return {**dict_, 'checksum': dhash.hexdigest()}
-
     def _add_uid(self, dict_: Dict, uid: Any) -> Dict:
         return {**dict_, 'uid': f'{uid}'}
 
@@ -183,11 +181,10 @@ class ElasticDataSource(
 
     def _action_for_upsert(self, index: str, objects: Iterable[DataObject], id_func: Callable,
                            field_prefix: str):
+        real_index_name = self._get_indices().get(index)
         for object_ in objects:
             obj = self._convert_data_object_to_dict(object_)
             obj = self._convert_dates(obj)
-            obj = self._add_checksum(obj)
-            obj = self._add_updated(obj)
             obj = self._prefix_fields(obj, field_prefix)
             obj = self._stringify_ids(obj)
             uid = id_func(object_)
@@ -196,7 +193,7 @@ class ElasticDataSource(
                 '_op_type': 'update',
                 'scripted_upsert': True,
                 'upsert': {},
-                '_index': index,
+                '_index': real_index_name,
                 '_id': uid,
                 'script': {
                     'source': self._upsert_script,
@@ -239,7 +236,7 @@ class ElasticDataSource(
         **kwargs
     ) -> None:
 
-        index = self.__get_index(object_type)
+        index = self.__get_index_or_alias(object_type)
         (no_of_operations, no_of_errors) = \
             self.helpers.bulk(self.es,
                               self._action_for_upsert(index,
@@ -264,14 +261,15 @@ class ElasticDataSource(
         # This tries to find an object in the DataSource that matches
         # the candidate key. If found it will perform the update
 
-        index = self.__get_index(object_type)
+        index = self.__get_index_or_alias(object_type)
+        real_index_name = self._get_indices().get(index)
         for (_, update) in updates:
             # We can get the candidate key dynamically from the actual update
             if 'candidate_key_func' in kwargs:
                 candidate_key = kwargs['candidate_key_func'](update)
             self.es.update_by_query(
-                index=index,
-                body=self._action_for_update(index,
+                index=real_index_name,
+                body=self._action_for_update(object_type,
                                              update,
                                              field_prefix,
                                              candidate_key),
@@ -351,19 +349,18 @@ class ElasticDataSource(
         """
         return s.replace('\n', ' ')
 
-    def _action_for_update(self, index: str, update: Dict,
+    def _action_for_update(self, object_type: str, update: Dict,
                            field_prefix: str, candidate_key: Iterable[str]):
         u = self._convert_dates(update)
-        u = self._add_checksum(u)
-        u = self._add_updated(u)
         f = DataSourceFilter()
         f.and_ = {}
         for key in candidate_key:
             # Don't want key in the upsert as it cannot change anyway
             f.and_[key] = {'eq': {'value': u.pop(key)}}
         u = self._prefix_fields(u, field_prefix)
+        u = self._convert_data_objects_in_update_to_dict(u)
         query = self._build_elasticsearch_query(
-            self.__get_object_type(index),
+            object_type,
             object_filters=f)
         return {
             'query': query,
@@ -376,7 +373,7 @@ class ElasticDataSource(
             },
         }
 
-    def __get_index(self, object_type: str) -> str:
+    def __get_index_or_alias(self, object_type: str) -> str:
         return f'{self.index_prefix}-{kebabcase(object_type)}'
 
     def __get_object_type(self, index: str) -> str:
@@ -460,7 +457,8 @@ class ElasticDataSource(
         search_after: list[Any] | None = None
     ) -> dict[str, Any]:
 
-        index = self.__get_index(object_type)
+        index = self.__get_index_or_alias(object_type)
+        real_index_name = self._get_indices().get(index)
         query = self._build_elasticsearch_query(object_type, object_filters)
         sort = self._build_elasticsearch_sort(object_type, sort_by)
         fields = list(self.runtime_fields[object_type].keys()) \
@@ -473,7 +471,7 @@ class ElasticDataSource(
         return self.es.search(
             from_=from_,
             size=page_size,
-            index=index,
+            index=real_index_name,
             query=query,
             sort=sort,
             fields=fields,
@@ -665,14 +663,15 @@ class ElasticDataSource(
         session: OperableSession | None = None,
         **kwargs
     ) -> Iterable[DataObject]:
-        index = self.__get_index(object_type)
+        index = self.__get_index_or_alias(object_type)
+        real_index_name = self._get_indices().get(index)
         query = self._build_elasticsearch_query(object_type, object_filters)
         fields = list(self.runtime_fields[object_type].keys()) \
             if object_type in self.runtime_fields else None
         runtime_mappings = self.runtime_fields[object_type] \
             if object_type in self.runtime_fields else None
         generator = self.helpers.scan(self.es,
-                                      index=index,
+                                      index=real_index_name,
                                       scroll='10m',
                                       size=500,
                                       query={'query': query},
@@ -683,7 +682,7 @@ class ElasticDataSource(
     def _convert_dict_to_data_objects(self, objs: Dict) -> Iterable:
         for obj in objs:
             if '_source' in obj:
-                type_ = self.__get_object_type(obj['_index'])
+                type_ = self.__real_index_to_object_type(obj['_index'])
                 id_ = obj['_id']
                 attributes = obj['_source']
                 runtime_attributes = obj['fields'] if 'fields' in obj else {}
@@ -767,7 +766,8 @@ class ElasticDataSource(
         object_filters: DataSourceFilter = None,
         **kwargs
     ) -> Dict:
-        index = self.__get_index(object_type)
+        index = self.__get_index_or_alias(object_type)
+        real_index_name = self._get_indices().get(index)
         query = self._build_elasticsearch_query(object_type, object_filters)
         fields = list(self.runtime_fields[object_type].keys()) \
             if object_type in self.runtime_fields else None
@@ -775,7 +775,7 @@ class ElasticDataSource(
             if object_type in self.runtime_fields else None
         resp = self.es.search(
             size=0,
-            index=index,
+            index=real_index_name,
             query=query,
             aggregations=aggregations,
             fields=fields,
@@ -1071,7 +1071,8 @@ class ElasticDataSource(
         object_filters: DataSourceFilter = None,
         **kwargs
     ) -> int:
-        index = self.__get_index(object_type)
+        index = self.__get_index_or_alias(object_type)
+        real_index_name = self._get_indices().get(index)
         query = self._build_elasticsearch_query(object_type, object_filters)
         fields = list(self.runtime_fields[object_type].keys()) \
             if object_type in self.runtime_fields else None
@@ -1079,7 +1080,7 @@ class ElasticDataSource(
             if object_type in self.runtime_fields else None
         # We are not using es.count so that we can use runtime fields
         resp = self.es.search(
-            index=index,
+            index=real_index_name,
             track_total_hits=True,
             size=0,
             query=query,
@@ -1088,13 +1089,34 @@ class ElasticDataSource(
         )
         return resp['hits']['total']['value']
 
-    @property
     @ttl_cache(ttl=3600)
+    def _get_indices(self) -> dict[str, str]:
+        # Get all as the actual indexes may not have the correct prefix
+        results = self.es.indices.get_alias('*')
+        aliased_indexes = {
+            alias: index
+            for index, aliases in results.items()
+            for alias in aliases.get('aliases', {}).keys()
+            if alias.startswith(self.index_prefix)
+        }
+        # Non-aliased indexes
+        non_aliased_indexes = {
+            index: index
+            for index in results.keys()
+            if index.startswith(self.index_prefix)
+        }
+        return aliased_indexes | non_aliased_indexes
+
+    def __real_index_to_object_type(self, index: str) -> str:
+        aliases = self._get_indices()
+        alias = next((k for k, v in aliases.items() if v == index), None)
+        return self.__get_object_type(alias) if alias else None
+
+    @property
     def supported_types(self):
-        index_names = self.es.cat.indices(h='index', s='index').split()
+        indexes = self._get_indices()
         return [self.__get_object_type(index_name)
-                for index_name in index_names
-                if index_name.startswith(self.index_prefix)]
+                for index_name in indexes.keys()]
 
     def __map_type(self, type_: str) -> str:
         if type_ in ['text', 'keyword']:
@@ -1106,16 +1128,17 @@ class ElasticDataSource(
         return type_
 
     def _get_attribute_types_for_object_type(self, object_type: str) -> Dict:
-        index_name = self.__get_index(object_type)
-        mapping = self.es.indices.get_mapping(index=index_name)
-        if 'properties' not in mapping[index_name]['mappings']:
+        index_or_alias_name = self.__get_index_or_alias(object_type)
+        real_index_name = self._get_indices().get(index_or_alias_name)
+        mapping = self.es.indices.get_mapping(index=index_or_alias_name)
+        if 'properties' not in mapping[real_index_name]['mappings']:
             return {}
-        properties = mapping[index_name]['mappings']['properties']
+        properties = mapping[real_index_name]['mappings']['properties']
         standard_types = {
             property_name: self.__map_type(properties[property_name]['type'])
             for property_name in properties
             if 'type' in properties[property_name]
-            and 'type' != 'uid'
+            and property_name != 'uid'
         }
         runtime_types = {
             name: self.__map_type(self.runtime_fields[object_type][name]['type'])
